//--------------------------------------------------------------------------------------------------
//	Author:	P.Roman, Curtea de Arges, Romania
//
// 	Date		  Version	Description
//	------------------------------------------------------------------------------------------------
//  20220217  1.0.0   First release
//  20220219  1.1.0   Modified parameterlist of constructor. Delayed reaction of sensor implemented.
//
//	ARDUINO test program for testing a IR-sensor with blinking IR-led.
//  With the parameters extraOnCycles and extraOffCycles the number of consecutive positive or
//  negative detections required to switch the sensor On or Off respectively, can be modified.
//  This minimizes environmental influences and effectively implements a delay on the sensor
//  reporting On or Off respectively.
//
//  ------------------------------------------------------------------------------------------------
//  PINS:
//
//  reserved        0 // reserved for Serial communication
//  reserved        1 // reserved for Serial communication
//  pinSenor       A0 // must be analog pin
//  pinLed          2 // could be any pin
//  pinOut          8 // could be any pin
//--------------------------------------------------------------------------------------------------
#define DEBUG	// comment out if serial output is no longer needed for debug purposes

#include <IRSensor.h>

//==================================================================================================
//======== User may change the parameters below ====================================================
//==================================================================================================
const byte pinSensor        = A0;	// Pin Sensor (must be an analog pin).
const byte pinLed           = 2;	// IR-led can be connected to any pin.
const byte pinOut           = 8;	// Output channel can be any pin.
const byte treshold         = 90;	// positive readings must be less than 90% of negative readings
const byte period           = 20;	// led on/off cycle [ms]
const byte extraOnCycles    = 4;	// delay  4*20 =   80 ms before reporting sensor state ON
const byte extraOffCycles   = 50;	// delay 50*20 = 1000 ms before reporting sensor state OFF 
//==================================================================================================

IRSensor theIRSensor(pinLed, pinSensor, treshold, extraOnCycles, extraOffCycles, period);

#ifdef DEBUG
void serialReport() {
  const unsigned long refreshInterval = 1000;  // refresh serial output every 1 second
  static unsigned long lastRefresh = 0;
  unsigned long now = millis();
  if ((now - lastRefresh) > refreshInterval) {
    if (theIRSensor.hasRisen()) {
      Serial.println(F("IR-sensor has risen"));
    }
    if (theIRSensor.hasFallen()) {
      Serial.println(F("IR-sensor has fallen"));
    }
    if (theIRSensor.isON()) {
      Serial.println(F("IR-sensor is now ON"));
    }
    else {
      Serial.println(F("IR-sensor is now OFF"));
    }
    lastRefresh = now;
  }
} 
#endif

void setup() {
#ifdef DEBUG
  Serial.begin(115200);
  Serial.println(F("-----------------------------------------"));
  Serial.println(F("----- Arduino Test IR-Sensor 1.1.0 ------"));
  Serial.println(F("-----------------------------------------"));
  Serial.println(F(""));
#endif
  pinMode(pinOut, OUTPUT);
  digitalWrite(pinOut, HIGH); // active LOW, so initially HIGH
} 

void loop() {
	theIRSensor.update();
  digitalWrite(pinOut, !theIRSensor.isON()); // active LOW
#ifdef DEBUG
  serialReport();
#endif
}
