#include <IRSensor.h>

//#define DEBUG_IRSENSOR	// comment out if serial output is no longer needed for debugging purposes

IRSensor::IRSensor(byte pinIRLed, byte pinIRSensor, byte treshold, byte extraOnCycles, byte extraOffCycles, byte period):
			m_pinIRLed(pinIRLed), m_pinIRSensor(pinIRSensor), m_treshold(treshold),
			m_stateActive(false), m_hasRisen(false), m_hasFallen(false) {

	m_samplingInterval = period / 2;
	m_stateLed       	 = false;																					// IR-led off
	pinMode(m_pinIRLed, OUTPUT); digitalWrite(m_pinIRLed, m_stateLed);	// IR-led off
	m_referenceValue   = analogRead(m_pinIRSensor);											// read IR-sensor
	m_minPosDetections = 1 + extraOnCycles;															// required number of consecutive positive detections to switch on
	m_minNegDetections = 1 + extraOffCycles;														// required number of consecutive negative detections to switch off
	m_PosDetections    = 0;																							// no consecutive positive detections yet
	m_NegDetections    = 0;
	m_lastSampling     = 0;
#ifdef DEBUG_IRSENSOR
	m_samplingInterval = 250;	// Make the samplingInterval 250 ms, for printing purposes
#endif
}	

void IRSensor::update() {
//--------------------------------------------------------------------------------------------------
//	This method must be called as often as possible from the main loop().
//--------------------------------------------------------------------------------------------------
	unsigned long now = millis();
	int newValue = 0;
	
	if ((now - m_lastSampling) > m_samplingInterval) {											// time for action
		if (m_stateLed) {																											// IR-Led is on
			newValue = analogRead(m_pinIRSensor);																// value of IR-sensor when IR-led is on
			if (100.0 * newValue < (float)m_treshold * m_referenceValue) {			// positive detection
				if (m_PosDetections < m_minPosDetections) {												// required number not reached
					m_PosDetections++;
				}
				else {																														// required number reached
					m_NegDetections = m_minNegDetections;														// set counter for delayed release
				}
			}
			else {																															// negative detection
				if (m_NegDetections > 1) {																				// still more negative detections required
					m_NegDetections--;																							// count down for release
				}
				else {																														// required number of negative detections reached
					m_PosDetections = 0;																						// reset number of positive detections
				}
			}
		}
		else {																																// IR-Led is off
			m_referenceValue = analogRead(m_pinIRSensor);												// value of IR-sensor when IR-led is off
		}

#ifdef DEBUG_IRSENSOR
		Serial.print(now); Serial.print("  "); Serial.print(m_stateLed); Serial.print("  ");
		if (m_stateLed) Serial.print(newValue); else Serial.print(m_referenceValue);
		Serial.print("  "); Serial.print(m_PosDetections); Serial.print("  "); Serial.println(m_NegDetections);		
#endif

		m_stateLed = !m_stateLed;																							// toggle Led
		digitalWrite(m_pinIRLed, m_stateLed);
		m_lastSampling = now;																									// register the time of this sampling
		
// update overall IR-sensor state for the outside world		
		
		if (m_PosDetections == m_minPosDetections) {
			if (!m_stateActive) {
				m_stateActive = true;  
				m_hasRisen    = true;
			}
		}
		else {
			if (m_stateActive) {
				m_stateActive = false;
				m_hasFallen   = true;
			}
		}
	}
}

bool IRSensor::isON() {
	return (m_stateActive);
}

bool IRSensor::isOFF() {
    return (!this->isON());
}

bool IRSensor::hasRisen() {
  bool returnValue = m_hasRisen;
  m_hasRisen = false;
  return (returnValue);
}

bool IRSensor::hasFallen() {
  bool returnValue = m_hasFallen;
  m_hasFallen = false;
  return (returnValue);
}
