#ifndef IRSENSOR_H
#define IRSENSOR_H

//--------------------------------------------------------------------------------------------------
//	Author:	P.Roman, Romania
//
// 	Date		Version	Description
//	------------------------------------------------------------------------------------------------
//	20220217	1.0.0	First release
//	20220221	1.1.0	Delayed On/Off of sensor implemented.
// 
//	Implementation of a IR-sensor with a blinking IR-Led.
//	This way a simple IR-sensor (e.g. TCRT5000) can be used reliably under different ambient light
//	conditions.
//
//	Restrictions:
//
//	The IR-Sensor must be connected to one of the Analog inputs [A0-A5].
//
//	Basic technique:
//
//	The IR-led is blinking at a fixed rate. The IR-sensor is read twice per cycle: once when the
//	IR-led is on, and once when the IR-led is off.
//	A negative detection is made when both readings are more or less identical.
//	A positive detection is made when both readings are significantly different from each other.
//	The sensor reports the state ON after (1+extraOnCyles) consecutive positive detections.
//	If extraOnCycles equals zero there is no delay on reporting the ON state. 	
//	The sensor reports the state OFF after (1+extraOffCycles) consecutive negative detections.	
//	If extraOffCycles equals zero there is no delay on reporting the OFF state. 	
//--------------------------------------------------------------------------------------------------

#include <Arduino.h>

class IRSensor {

	public:
		IRSensor(byte pinIRLed, byte pinIRSensor, byte treshold = 90, byte extraOnCycles = 0, byte extraOffCycles = 0, byte period = 20);
		void update();											// must be called as often as possible from the main loop()
    bool isON();
    bool isOFF();
		bool hasRisen();
		bool hasFallen();

	private:
		byte          m_pinIRLed;						// IR-led pin.
		byte          m_pinIRSensor;				// IR-sensor pin must one of the analog input pins.
		byte          m_samplingInterval;		// half of the full cycle period [ms].
		byte					m_treshold;						// percentage of the analog value of the reference state (Led off)
																				// for the IR-sensor to make a positive detection.
		byte					m_minPosDetections;		// required number of consecutive positive detections to enter state ON.
		byte					m_minNegDetections;		// required number of consecutive negative detections to return to state OFF.

		unsigned long	m_lastSampling;
		bool					m_stateLed;						// IR-led on or off
		int						m_referenceValue;			// reading when IR-led is off
		byte					m_PosDetections;			// current number of consecutive positive detections (count-up
		byte					m_NegDetections;			// current number of consecutive negative detections (count-down)
		
		bool					m_stateActive;				// reported to the outside world
		bool					m_hasRisen;						// reported to the outside world
		bool					m_hasFallen;					// reported to the outside world
};

#endif
